<!-- Stylesheet to pretty-print XML with nested indents and CRs -->
<!-- Created by Philip Mansfield April 19, 2000 -->

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" indent="no" media-type="text/xml"/>

<!-- The following instruction will strip all pure whitespace nodes from the source tree except those which inherit xml:space="preserve" -->
<xsl:strip-space elements="*"/>

<!--
   The seven node types are copied from source to result tree as follows: 
   Root - always generated in result tree; no need to copy
   Element, PI and Comment - indented and copied with an explicit template (below)
   Attribute - copied with an explicit template (below)
   Namespace - automatically copied along with its parent element
   Text - copied with an explicit template (below) that removes extraneous white space
-->

<xsl:template name="indent">
<xsl:text>
</xsl:text>
   <xsl:for-each select="ancestor::*">
      <xsl:text> </xsl:text>
   </xsl:for-each>
</xsl:template>
     
<xsl:template match="*|processing-instruction()|comment()">
   <!-- Start tag can be safely indented as long as context node is not part of mixed content: -->
   <xsl:if test="not(../text())">
      <xsl:call-template name="indent"/>
   </xsl:if>
   <xsl:choose>
      <!-- When there are no children, allow abbreviated tag end: -->
      <xsl:when test="not(node())"><xsl:copy><xsl:apply-templates select="@*"/></xsl:copy></xsl:when>
      <xsl:otherwise>
         <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
            <!-- Indent closing tag unless context node has text content itself: -->
            <xsl:if test="not(text())">
               <xsl:call-template name="indent"/>
            </xsl:if>
         </xsl:copy>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="@*">
   <xsl:copy/>
</xsl:template>

<xsl:template match="text()">
   <xsl:choose>
      <!-- The nearest inherited xml:space attribute determines whether we can normalize white space -->
      <xsl:when test="ancestor-or-self::*[@xml:space][1]/@xml:space='preserve'">
         <xsl:value-of select="."/>
      </xsl:when>
      <xsl:otherwise>
         <xsl:value-of select="normalize-space()"/>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

</xsl:stylesheet>
