/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache\@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation, and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.ibm.com .  For more information
 * on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 * $Id: DOMWrite.cpp,v 1.2 2004/02/09 19:01:57 johanjan Exp $
 */


// ---------------------------------------------------------------------------
//  This sample program which invokes the DOMParser to build a DOM tree for
//  the specified input file. It then walks the tree, and prints out the data
//  as an XML file.
//
//   Limitations:
//      1.  The encoding="xxx" clause in the XML header should reflect
//          the system local code page, but does not.
//      2.  Cases where the XML data contains characters that can not
//          be represented in the system local code page are not handled.
//      3.  Enabled namespace processing won't affect the output, since
//          DOM doesn't do namespace yet. But it will confirm that all
//          prefixes are correctly mapped, else you'll get errors.
// ---------------------------------------------------------------------------


// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <xercesc/util/PlatformUtils.hpp>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMWriter.hpp>

#include <xercesc/framework/StdOutFormatTarget.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/XMLUni.hpp>
#include "ko/DOMTreeErrorReporter.hpp"
#include <string.h>
#include <stdlib.h>


#include "ko/DOMTreeErrorReporter.hpp"
#include "ko/DOMPrintFilter.hpp"
#include "ko/DOMPrintErrorHandler.hpp"

#include "ko/DOMWrite.hpp"



void writeDocument(ostream &os, DOM_Document &doc)
{
  try
  {
    // get a serializer, an instance of DOMWriter
    XMLCh tempStr[100];
    XMLString::transcode("LS", tempStr, 99);
    DOMImplementation *impl =
      DOMImplementationRegistry::getDOMImplementation(tempStr);
    DOMWriter *theSerializer =
      ((DOMImplementationLS*)impl)->createDOMWriter();

    // set user specified end of line sequence and output encoding
    theSerializer->setNewLine(gMyEOLSequence);
    theSerializer->setEncoding(gOutputEncoding);

    // plug in user's own filter
    if (gUseFilter)
    {
      // even we say to show attribute, but the DOMWriter
      // will not show attribute nodes to the filter as
      // the specs explicitly says that DOMWriter shall
      // NOT show attributes to DOMWriterFilter.
      //
      // so DOMNodeFilter::SHOW_ATTRIBUTE has no effect.
      // same DOMNodeFilter::SHOW_DOCUMENT_TYPE, no effect.
      //
      myFilter = new DOMPrintFilter(DOMNodeFilter::SHOW_ELEMENT   |
				    DOMNodeFilter::SHOW_ATTRIBUTE |
				    DOMNodeFilter::SHOW_DOCUMENT_TYPE
				    );
      theSerializer->setFilter(myFilter);
    }

    // plug in user's own error handler
    DOMErrorHandler *myErrorHandler = new DOMPrintErrorHandler();
    theSerializer->setErrorHandler(myErrorHandler);

    // set feature if the serializer supports the feature/mode
    if (theSerializer->canSetFeature(XMLUni::fgDOMWRTSplitCdataSections,
				     gSplitCdataSections))
      theSerializer->setFeature(XMLUni::fgDOMWRTSplitCdataSections,
				gSplitCdataSections);
			
    if (theSerializer->canSetFeature(XMLUni::fgDOMWRTDiscardDefaultContent,
				     gDiscardDefaultContent))
      theSerializer->setFeature(XMLUni::fgDOMWRTDiscardDefaultContent,
				gDiscardDefaultContent);

    if (theSerializer->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint,
				     gFormatPrettyPrint))
      theSerializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint,
				gFormatPrettyPrint);

    //
    // Plug in a format target to receive the resultant
    // XML stream from the serializer.
    //
    // StdOutFormatTarget prints the resultant XML stream
    // to stdout once it receives any thing from the serializer.
    //
    XMLFormatTarget *myFormTarget = new StdOutFormatTarget();

    // get the DOM representation
    DOMNode *doc = parser->getDocument();

    //
    // do the serialization through DOMWriter::writeNode();
    //
    theSerializer->writeNode(myFormTarget, *doc);

    delete theSerializer;

    //
    // Filter, formatTarget and error handler
    // are NOT owned by the serializer.
    //
    delete myFormTarget;
    delete myErrorHandler;

    if (gUseFilter)
      delete myFilter;      	

  }
  catch (XMLException& e)
  {
    cerr << "An error occurred during creation of output transcoder. Msg is:"
	 << endl
	 << StrX(e.getMessage()) << endl;
    retval = 4;
  }
}
