#include <stdlib.h>
#include <assert.h>

#include <strstream>
#include <iostream>
#include <iomanip>

/*
#include <xercesc/util/PlatformUtils.hpp>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMWriter.hpp>

#include <xercesc/framework/StdOutFormatTarget.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/XMLUni.hpp>

#include "ko/DOMTreeErrorReporter.hpp"

#include "ko/DOMTreeErrorReporter.hpp"
#include "ko/DOMPrintFilter.hpp"
#include "ko/DOMPrintErrorHandler.hpp"
#include "ko/OStreamFormatTarget.hpp"

//#include "ko/DOMWrite.hpp"
*/
#include "ko/PhysicalDOMTranslator.hpp"
#include "ko/MountedPoint.hpp"
#include "ko/MechanicsModel.hpp"

using namespace xercesc;
using namespace std;

namespace ko
{

PhysicalDOMTranslator::PhysicalDOMTranslator()
{
  rootTag = "physical";
}


void PhysicalDOMTranslator::translate(const DOMElement &node, bool &target)
{
  DOMText *textnode = (DOMText *)node.getFirstChild();

  assert(textnode->getNodeType() == DOMNode::TEXT_NODE);  

  char *text = XMLString::transcode(textnode->getData());
  
  if(!strcmp("true", text))
    target = true;
  else
    target = false;

  delete [] text;
}

void PhysicalDOMTranslator::translate(const DOMElement &node, double &target)
{
  DOMText *textnode = (DOMText *)node.getFirstChild();

  assert(textnode->getNodeType() == DOMNode::TEXT_NODE);  

  char *text = XMLString::transcode(textnode->getData());
  
  target = strtod(text, NULL);

  delete [] text;
}

void PhysicalDOMTranslator::translate(const DOMElement &node, std::string &target)
{
  DOMText *textnode = (DOMText *)node.getFirstChild();

  assert(textnode->getNodeType() == DOMNode::TEXT_NODE);  

  char *text = XMLString::transcode(textnode->getData());

  target = text;

  delete [] text;
}

void PhysicalDOMTranslator::translate(const DOMElement &node, Element *&target)
{
  DOMText *textnode = (DOMText *)node.getFirstChild();

  assert(textnode->getNodeType() == DOMNode::TEXT_NODE);  

  char *text = XMLString::transcode(textnode->getData());
  
  std::string name = text;

  target = elementLookup[name];

  delete [] text;
}

void PhysicalDOMTranslator::translate(const DOMElement &node, Material *&target)
{
  DOMText *textnode = (DOMText *)node.getFirstChild();

  assert(textnode->getNodeType() == DOMNode::TEXT_NODE);  

  char *text = XMLString::transcode(textnode->getData());
  
  std::string name = text;

  target = materialLookup[name];

  delete [] text;
}

void PhysicalDOMTranslator::translate(const DOMElement &elementnode, Vector &target)
{
  dolfin::real a;
  translateChild(elementnode, "x", a);
  target(0) = a;
  translateChild(elementnode, "y", a);
  target(1) = a;
  translateChild(elementnode, "z", a);
  target(2) = a;
}

void PhysicalDOMTranslator::translate(const DOMElement &elementnode, Material &target)
{
  translateChild(elementnode, "name", target.name);
  translateChild(elementnode, "young", target.young);
  translateChild(elementnode, "density", target.density);
  translateChild(elementnode, "damp", target.damp);
  translateChild(elementnode, "friction", target.friction);
  translateChild(elementnode, "plasticity", target.plasticity);
  translateChild(elementnode, "strength", target.strength);
  translateChild(elementnode, "collisionStrength", target.collisionStrength);
}


void PhysicalDOMTranslator::translate(const DOMElement &elementnode, Element &target)
{
  translateChild(elementnode, "name", target.name);
  translateChild(elementnode, "materialName", target.material);
  translateChild(elementnode, "radius", target.radius);
  translateChild(elementnode, "position", target.position);
  translateChild(elementnode, "velocity", target.velocity);
  translateChild(elementnode, "force", target.force);
  translateChild(elementnode, "artificial", target.artificial);

  cerr << "e: " << target.name << endl;
  cerr << "mass: " << MechanicsModel::mass(target) << endl;
}

void PhysicalDOMTranslator::translate(const DOMElement &elementnode,
				      Connection &target)
{
  translateChild(elementnode, "element1", target.element1);
  translateChild(elementnode, "element2", target.element2);
  translateChild(elementnode, "length", target.length);
  translateChild(elementnode, "transient", target.transient);
  translateChild(elementnode, "tension", target.tension);
  translateChild(elementnode, "damage", target.damage);

//   cerr << "e1: " << target.element1->name << endl;
//   cerr << "e2: " << target.element2->name << endl;

//   cerr << "hooke: " << MechanicsModel::hooke(target) << endl;
//   cerr << "damp: " << MechanicsModel::damp(target) << endl;
}

void PhysicalDOMTranslator::translate(const DOMElement &elementnode, Physical &target)
{
//   std::cerr << "translate physical" << std::endl;

  //DOMNodeList childlist, patternlist;
  int listlen;


  XMLCh *tmpstr = XMLString::transcode("material");

  DOMNodeList *childlist =
    elementnode.getElementsByTagName(tmpstr);
  XMLString::release(&tmpstr);
  listlen = childlist->getLength();
  for(int i = 0; i < listlen; i++)
  {
    DOMElement *childnode = (DOMElement *)childlist->item(i);
      
    Material *m = new Material();
      
    translate(*childnode, *m);
    target.materials.push_back(m);

    materialLookup[m->name] = m;
  }

  tmpstr = XMLString::transcode("element");
  childlist =
    elementnode.getElementsByTagName(tmpstr);
  XMLString::release(&tmpstr);

  listlen = childlist->getLength();
  //for(int i = 0; i < childlist->getLength(); i++)
  for(int i = 0; i < listlen; i++)
  {
    DOMElement *childnode = (DOMElement *)childlist->item(i);
      
    Element &e = *(new Element());
      
    translate(*childnode, e);
    target.addElement(e);
    //target.elements.push_back(e);

    elementLookup[e.name] = &e;
  }

  tmpstr = XMLString::transcode("connection");
  childlist =
    elementnode.getElementsByTagName(tmpstr);
  XMLString::release(&tmpstr);

  listlen = childlist->getLength();
  for(int i = 0; i < listlen; i++)
  {
    DOMElement *childnode = (DOMElement *)childlist->item(i);
      
    Connection &c = *(new Connection());
      
    translate(*childnode, c);

    c.connect(*(c.element1), *(c.element2));

    //target.connections.push_back(&c);
    target.addConnection(c);
  }
}

template <class T>
void PhysicalDOMTranslator::translateChild(const DOMElement &node,
					   const char *name, T &target)
{
  XMLCh *tmpstr = XMLString::transcode(name);
  DOMNodeList *childlist =
    node.getElementsByTagName(tmpstr);
  XMLString::release(&tmpstr);

  if(childlist->getLength() != 0)
  {
    DOMElement *child = (DOMElement *)childlist->item(0);
      
    translate(*child, target);
  }
}

void PhysicalDOMTranslator::create(double &source, DOMElement &element,
				   DOMDocument &doc)
{
  ostrstream data;

  // 0.0e+00 is not recognized as a "double" by Xerces. This appears to be
  // non-conformant behavior.

  if(source != 0.0)
  {
    data.setf(ios::scientific);
    data.precision(20);
  }

  data << source << ends;

  char *result = data.str();

  XMLCh *tmpstr = XMLString::transcode(result);
  DOMText *newElem = doc.createTextNode(tmpstr);
  XMLString::release(&tmpstr);

  element.appendChild(newElem);

  delete [] result;
}

void PhysicalDOMTranslator::create(bool &source, DOMElement &element,
				   DOMDocument &doc)
{
  ostrstream data;

  if(source == true)
    data << "true" << ends;
  else
    data << "false" << ends;

  char *result = data.str();

  XMLCh *tmpstr = XMLString::transcode(result);
  DOMText *newElem = doc.createTextNode(tmpstr);
  XMLString::release(&tmpstr);

  element.appendChild(newElem);

  delete [] result;
}

void PhysicalDOMTranslator::create(std::string &source, DOMElement &element,
				   DOMDocument &doc)
{
  const char *name = source.c_str();

  XMLCh *tmpstr = XMLString::transcode(name);
  DOMText *newElem = doc.createTextNode(tmpstr);
  XMLString::release(&tmpstr);

  element.appendChild(newElem);
}


DOMDocument *PhysicalDOMTranslator::create(Physical &source)
{
  // get a serializer, an instance of DOMWriter
  XMLCh tempStr[100];
  XMLString::transcode("LS", tempStr, 99);

  DOMImplementation *impl =
    DOMImplementationRegistry::getDOMImplementation(tempStr);;
  
  
  XMLCh *tmpstr = XMLString::transcode("physical");
  DOMDocument *doc =
    impl->createDocument(
			 0,                    // root element namespace URI.
			 tmpstr,            // root element name
			 0);  // document type object (DTD).
  XMLString::release(&tmpstr);

  
  DOMElement *rootElem = doc->getDocumentElement();

  XMLCh *tmpstr1 = XMLString::transcode("xmlns:xsi");
  XMLCh *tmpstr2 = XMLString::transcode("http://www.w3.org/2001/XMLSchema-instance");

  rootElem->setAttribute(tmpstr1, tmpstr2);
  XMLString::release(&tmpstr1);
  XMLString::release(&tmpstr2);

  tmpstr1 = XMLString::transcode("xsi:noNamespaceSchemaLocation");
  tmpstr2 = XMLString::transcode("physical-schema.xml");

  rootElem->setAttribute(tmpstr1, tmpstr2);
  XMLString::release(&tmpstr1);
  XMLString::release(&tmpstr2);



  for(list<Material *>::iterator i = source.materials.begin();
      i != source.materials.end(); i++)
  {
    Material &m = **i;

    tmpstr = XMLString::transcode("material");
    DOMElement *newElem = doc->createElement(tmpstr);
    XMLString::release(&tmpstr);

    rootElem->appendChild(newElem);
    //allocElements.push_back(newElem);
      
    create(m, *newElem, *doc);
  }

  for(list<Element *>::iterator i = source.elements.begin();
      i != source.elements.end(); i++)
  {
    Element &e = **i;

    tmpstr = XMLString::transcode("element");
    DOMElement *newElem = doc->createElement(tmpstr);
    XMLString::release(&tmpstr);

    rootElem->appendChild(newElem);
    //allocElements.push_back(newElem);
      
    create(e, *newElem, *doc);
  }

  for(list<Connection *>::iterator i = source.connections.begin();
      i != source.connections.end(); i++)
  {
    Connection &c = **i;

    tmpstr = XMLString::transcode("connection");
    DOMElement *newElem = doc->createElement(tmpstr);
    XMLString::release(&tmpstr);
    rootElem->appendChild(newElem);
    //allocElements.push_back(newElem);
      
    create(c, *newElem, *doc);
  }

  return doc;
}

void PhysicalDOMTranslator::create(Vector &source, DOMElement &element,
				   DOMDocument &doc)
{
  dolfin::real a;
  a = source(0);
  createChild(a, element, "x", doc);
  a = source(1);
  createChild(a, element, "y", doc);
  a = source(2);
  createChild(a, element, "z", doc);
}

void PhysicalDOMTranslator::create(Material &source, DOMElement &element,
				   DOMDocument &doc)
{
  if(source.name != "")
    createChild(source.name, element, "name", doc);
  createChild(source.young, element, "young", doc);
  createChild(source.density, element, "density", doc);
  createChild(source.damp, element, "damp", doc);
  createChild(source.friction, element, "friction", doc);
  createChild(source.plasticity, element, "plasticity", doc);
  createChild(source.strength, element, "strength", doc);
  createChild(source.collisionStrength, element, "collisionStrength", doc);
}

void PhysicalDOMTranslator::create(Element &source, DOMElement &element,
				   DOMDocument &doc)
{
  if(source.name != "")
    createChild(source.name, element, "name", doc);

  createChild(source.material->name, element, "materialName", doc);

  createChild(source.radius, element, "radius", doc);
  createChild(source.position, element, "position", doc);
  createChild(source.velocity, element, "velocity", doc);
  createChild(source.force, element, "force", doc);
  createChild(source.artificial, element, "artificial", doc);
}

void PhysicalDOMTranslator::create(Connection &source, DOMElement &element,
				   DOMDocument &doc)
{
  createChild(source.element1->name, element, "element1", doc);
  createChild(source.element2->name, element, "element2", doc);
  createChild(source.length, element, "length", doc);
  createChild(source.transient, element, "transient", doc);
  createChild(source.tension, element, "tension", doc);
  createChild(source.damage, element, "damage", doc);
}

template <class T>
void PhysicalDOMTranslator::createChild(T &source, DOMElement &element,
					const char *name, DOMDocument &doc)
{
  XMLCh *tmpstr = XMLString::transcode(name);
  DOMElement *newElem = doc.createElement(tmpstr);
  XMLString::release(&tmpstr);
  create(source, *newElem, doc);
  element.appendChild(newElem);
  //allocElements.push_back(newElem);
}

}

