#if !defined(PHYSICALDOMTRANSLATOR_HPP)
#define PHYSICALDOMTRANSLATOR_HPP

#include <iostream>
#include <vector>
#include <string>
#include <map>

#include "ko/DOMParser.hpp"

#include "ko/LinAlg.hpp"
#include "ko/Material.hpp"
#include "ko/Element.hpp"
#include "ko/Connection.hpp"
#include "ko/Physical.hpp"

class DOMTreeErrorReporter;

namespace ko
{

class PhysicalDOMTranslator : public xercesc::DOMParser<Physical>
{
public:
  std::map<std::string, Element *> elementLookup;
  std::map<std::string, Material *> materialLookup;

  PhysicalDOMTranslator();

  void translate(const xercesc::DOMElement &node, bool &target);
  void translate(const xercesc::DOMElement &node, double &target);
  void translate(const xercesc::DOMElement &node, std::string &target);

  void translate(const xercesc::DOMElement &node, Element *&target);
  void translate(const xercesc::DOMElement &node, Material *&target);

  void translate(const xercesc::DOMElement &node, Vector &target);
  void translate(const xercesc::DOMElement &node, Material &target);
  void translate(const xercesc::DOMElement &node, Element &target);
  void translate(const xercesc::DOMElement &node, Connection &target);
  void translate(const xercesc::DOMElement &node, Physical &target);

  template <class T>
  void translateChild(const xercesc::DOMElement &node, const char *,
		      T &target);

  xercesc::DOMDocument *create(Physical &source);

  void create(double &source, xercesc::DOMElement &element,
	      xercesc::DOMDocument &doc);
  void create(bool &source, xercesc::DOMElement &element,
	      xercesc::DOMDocument &doc);
  void create(std::string &source, xercesc::DOMElement &element,
	      xercesc::DOMDocument &doc);
  void create(Vector &source, xercesc::DOMElement &element,
	      xercesc::DOMDocument &doc);
  void create(Material &source, xercesc::DOMElement &element,
	      xercesc::DOMDocument &doc);
  void create(Element &source, xercesc::DOMElement &element,
	      xercesc::DOMDocument &doc);
  void create(Connection &source, xercesc::DOMElement &element,
	      xercesc::DOMDocument &doc);

  template <class T>
  void createChild(T &source, xercesc::DOMElement &element, const char *, 
		   xercesc::DOMDocument &doc);
};

}

void print();

#endif // PhysicalDOMTranslator

