#include <stdio.h>
#include <math.h>
#include <list>
#include <iostream>

#include "ko/BSPTree.hpp"
#include "ko/LinAlg.hpp"
#include "ko/Element.hpp"

using namespace std;

namespace ko
{

  BSPTree::BSPTree( list<BSPTree *> *alloc_pool ) : center(3), max(3), min(3)
  {
    //  fprintf(stderr,"BSPTree::ctor\n");
    pool		= alloc_pool;

    lt_child	= NULL;
    geq_child	= NULL;
    parent	= NULL;
    level		= 0;
    orientation	= X_PARTITION;
  }

  BSPTree::~BSPTree()
  {
    //	fprintf(stderr,"BSPTree::dtor\n");
    if( lt_child != NULL )
      delete lt_child;
    if( geq_child != NULL )
      delete geq_child;

  }

  void BSPTree::Partition(list<Element *> &set)
  {
    if(set.empty())
      ;
    else if( set.end() == (++(set.begin())) )
    {
      // set contains one element

      //cerr << "Adding to data" << endl;
      data.splice(data.end(), set, set.begin());
    }
    else
    {
      //cerr << "Further partitioning " << level << " " << set.size() << endl;
      //cerr << "max " << max(0) << " " << max(1) << " " << max(2) << endl;
      //cerr << "min " << min(0) << " " << min(1) << " " << min(2) << endl;
      //min->Print();
      //max->Print();
      
      list<Element *>	lt_set, geq_set;
      
      for( list<Element *>::iterator it = set.begin();
	   it != set.end(); )
      {
	Element	&ce = **it;
	Vector	&cv = ce.position;
	list<Element *>::iterator curit = it++;

	switch( orientation )
	{
	case X_PARTITION:
	  {
	    if( fabs( cv(0) - center(0) ) < ce.radius )
	    {
	      data.splice(data.end(), set, curit);
	    }
	    else
	    {
	      if( cv(0) < center(0) )
	      {
		lt_set.splice(lt_set.end(), set, curit);
	      }
	      else
	      {
		geq_set.splice(geq_set.end(), set, curit);
	      }
	    }
	  }
	  break;
	case Y_PARTITION:
	  {
	    if( fabs( cv(1) - center(1) ) < ce.radius )
	    {
	      data.splice(data.end(), set, curit);
	    }
	    else
	    {
	      if( cv(1) < center(1) )
	      {
		lt_set.splice(lt_set.end(), set, curit);
	      }
	      else
	      {
		geq_set.splice(geq_set.end(), set, curit);
	      }
	    }
	  }
	  break;
	case Z_PARTITION:
	  {
	    if( fabs( cv(2) - center(2) ) < ce.radius )
	    {
	      data.splice(data.end(), set, curit);
	    }
	    else
	    {
	      if( cv(2) < center(2) )
	      {
		lt_set.splice(lt_set.end(), set, curit);
	      }
	      else
	      {
		geq_set.splice(geq_set.end(), set, curit);
	      }
	    }
	  }
	  break;
	default:
	  fprintf(stderr,"shouldn't be here\n");
	  break;
	}
      }

      switch( orientation )
      {
      case X_PARTITION:
	{
	  lt_child = pool_alloc_bsptree( pool );
	  lt_child->orientation 	= Y_PARTITION;
	  lt_child->level		= level+1;
	  lt_child->parent		= this;
	  lt_child->min = min;
	  lt_child->max = max;
	  lt_child->max(0) = center(0);
	  lt_child->computeCenter();
	  if( !lt_set.empty() )
	    lt_child->Partition(lt_set);

	  geq_child = pool_alloc_bsptree( pool );
	  geq_child->orientation 	= Y_PARTITION;
	  geq_child->level		= level+1;
	  geq_child->parent		= this;
	  geq_child->min = min;
	  geq_child->max = max;
	  geq_child->min(0) = center(0);
	  geq_child->computeCenter();
	  if( !geq_set.empty() )
	    geq_child->Partition(geq_set);
	}
	break;
      case Y_PARTITION:
	{
	  lt_child = pool_alloc_bsptree( pool );
	  lt_child->orientation 	= Z_PARTITION;
	  lt_child->level		= level+1;
	  lt_child->parent		= this;
	  lt_child->min = min;
	  lt_child->max = max;
	  lt_child->max(1) = center(1);
	  lt_child->computeCenter();
	  if( !lt_set.empty() )
	    lt_child->Partition(lt_set);

	  geq_child = pool_alloc_bsptree( pool );
	  geq_child->orientation 	= Z_PARTITION;
	  geq_child->level		= level+1;
	  geq_child->parent		= this;
	  geq_child->min = min;
	  geq_child->max = max;
	  geq_child->min(1) = center(1);
	  geq_child->computeCenter();
	  if( !geq_set.empty() )
	    geq_child->Partition(geq_set);
	}
	break;
      case Z_PARTITION:
	{
	  lt_child = pool_alloc_bsptree( pool );
	  lt_child->orientation 	= X_PARTITION;
	  lt_child->level		= level+1;
	  lt_child->parent		= this;
	  lt_child->min = min;
	  lt_child->max = max;
	  lt_child->max(2) = center(2);
	  if( !lt_set.empty() )
	    lt_child->Partition(lt_set);

	  geq_child = pool_alloc_bsptree( pool );
	  geq_child->orientation 	= X_PARTITION;
	  geq_child->level		= level+1;
	  geq_child->parent		= this;
	  geq_child->min = min;
	  geq_child->max = max;
	  geq_child->min(2) = center(2);
	  if( !geq_set.empty() )
	    geq_child->Partition(geq_set);
	}
	break;
      default:
	fprintf(stderr,"shouldn't be here\n");
	break;
      }
    }
  }

  void BSPTree::Collide( Element &e, list<Element *> &collisions )
  {
    Vector	&cv = e.position;

//     std::cout << "collide: " << "data: " << data.size() << " level: " <<
//       level << std::endl;

    for( list<Element *>::iterator it = data.begin();
	 it != data.end() ; )
    {
      Element &e2 = **it;
      list<Element *>::iterator curit = it++;
    
      if( &e2 == &e )
      {
 	data.erase(curit);
      }
      else
      {
	collisions.push_back(&e2);
      }
    }
  
    switch( orientation )
    {
    case X_PARTITION:
      {
	if( fabs( cv(0) - center(0) ) < e.radius )
	{
	  if( lt_child != NULL )
	    lt_child->Collide( e, collisions );
	  if( geq_child != NULL )
	    geq_child->Collide( e, collisions );
	}
	else
	{
	  if( cv(0) < center(0) )
	  {
	    if( lt_child != NULL )
	    {
	      lt_child->Collide( e, collisions );
	    }
	  }
	  else
	  {
	    if( geq_child != NULL )
	    {
	      geq_child->Collide( e, collisions );
	    }
	  }
	}
      }
      break;
    case Y_PARTITION:
      {
	if( fabs( cv(1) - center(1) ) < e.radius )
	{
	  if( lt_child != NULL )
	    lt_child->Collide( e, collisions );
	  if( geq_child != NULL )
	    geq_child->Collide( e, collisions );
	}
	else
	{
	  if( cv(1) < center(1) )
	  {
	    if( lt_child != NULL )
	    {
	      lt_child->Collide( e, collisions );
	    }
	  }
	  else
	  {
	    if( geq_child != NULL )
	    {
	      geq_child->Collide( e, collisions );
	    }
	  }
	}
      }
      break;
    case Z_PARTITION:
      {
	if( fabs( cv(2) - center(2) ) < e.radius )
	{
	  if( lt_child != NULL )
	    lt_child->Collide( e, collisions );
	  if( geq_child != NULL )
	    geq_child->Collide( e, collisions );
	}
	else
	{
	  if( cv(2) < center(2) )
	  {
	    if( lt_child != NULL )
	    {
	      lt_child->Collide( e, collisions );
	    }
	  }
	  else
	  {
	    if( geq_child != NULL )
	    {
	      geq_child->Collide( e, collisions );
	    }
	  }
	}
      }
      break;
    default:
      fprintf(stderr, "shouldn't be here\n" );
      break;
    }
  }


  BSPTree *BSPTree::pool_alloc_bsptree( list<BSPTree *> *pool )
  {
    BSPTree	*ret_tree;
	
    if( !pool->empty() )
    {
      ret_tree = (BSPTree *)pool->front();
      pool->remove( pool->front() );

      ret_tree->data.clear();
      ret_tree->lt_child		= NULL;
      ret_tree->geq_child		= NULL;
      ret_tree->level                   = 0;
    }
    else
    {
      ret_tree = new BSPTree( pool );
    }

    return ret_tree;
  }

  void BSPTree::pool_free_bsptree( BSPTree *tree, list<BSPTree *> *pool)
  {
    pool->push_back( tree );
    if( tree->lt_child != NULL )
      pool_free_bsptree( tree->lt_child, pool );
    if( tree->geq_child != NULL )
      pool_free_bsptree( tree->geq_child, pool );
  }

  void BSPTree::computeCenter()
  {
    center = min;
    center += max;
    center *= 0.5;
  }

}


