#include <stdio.h>
#include <iostream>

#include "ko/IntersectionDetector.hpp"

using namespace std;

namespace ko
{

IntersectionDetector::IntersectionDetector()
{
  spacetree1_pool = new list<BSPTree *>();
  spacetree1	  = new BSPTree( spacetree1_pool );
}

void IntersectionDetector::init(list<Element *> &targets)
{
  elements.clear();
  elements = targets;

  buildTrees();
}
 


void IntersectionDetector::buildTrees()
{
  BSPTree::pool_free_bsptree( spacetree1, spacetree1_pool );

  spacetree1 = BSPTree::pool_alloc_bsptree( spacetree1_pool );

  Vector min(3), max(3), center(3);

  list<Vector *> positions;

  for( list<Element *>::iterator it = elements.begin();
       it != elements.end(); it++)
  {
    Element &e = **it;

    positions.push_back(&(e.position));
  }


  bbox(positions, min, max);

  center = min;
  center += max;
  center *= 0.5;

  spacetree1->min = min;
  spacetree1->max = max;
  spacetree1->center = center;

  list<Element *> colliders = elements;

  spacetree1->Partition(colliders);
}

void IntersectionDetector::intersectionList(Element &e1, list<Element *> &intersections)
{
//   std::cout << "ilist: " << e1.index << endl;

  list<Element *> potentials;

  spacetree1->Collide(e1, potentials);

  for(list<Element *>::iterator it = potentials.begin();
      it != potentials.end(); it++)
  {
    Element &e2 = **it;

//     std::cout << "potentials: " << e2.index << endl;
      
    if(&e1 != &e2 && !e1.isConnected(e2) && isIntersection(e1, e2))
    {
      intersections.push_back(&e2);
    }
  }
}

bool IntersectionDetector::isIntersection(Element &e1, Element &e2)
{
   /*
   double rnorm;

   Vector r(3);

   add(e1.position, scaled(e2.position, -1), r);

   rnorm = two_norm(r);

   if(rnorm <= (e1.radius + e2.radius))
   {
     Vector vrel(3);

     double vdir;

     add(e1.velocity, scaled(e2.velocity, -1), vrel);

     // project vrel onto connection
     add(vrel, scaled(r, (dot(vrel, r) / (rnorm * rnorm))), vrel);

     // how do the velocities point?
     vdir = dot(r, vrel);

     if(e1.name == "element 1000" || e2.name == "element 1000")
       cerr << "vdir: " << vdir << endl;

     // If the velocities are pointing away from each other, we say there
     // is no collision.
     if(vdir < 0)
     {
       return true;
     }
   }
   */
   double rnorm;

   Vector r(3);

   r = e1.position;
   r.axpy(-1, e2.position);
   rnorm = r.norm();

   if(rnorm <= (e1.radius + e2.radius))
   {
     return true;
   }

   return false;
 }

}
