#include <iostream>

#include "ko/MechanicsModel.hpp"

using namespace std;

namespace ko
{

MechanicsModel::MechanicsModel() : g(3)
{
  g(1) = -9.81;
}

void MechanicsModel::force(Element &e, Vector &f)
{
  static Vector eforce(3), cforce(3), sforce(3);
  static Vector tmpf(3);

  f = 0;

//   cout << "f: " << endl;
//   f.disp();


  double mass = e.material->density * pow(2 * e.radius, 3);

  //std::cerr << "mass: " << mass << endl;

  for(list<Connection *>::iterator it = e.connections.begin();
      it != e.connections.end(); it++)
  {
    Connection &c = **it;
    
    force(c, e, tmpf);
      
    f += tmpf;
  }	  
  //eforce = cforce;
  f.axpy(mass, g);
}

void MechanicsModel::force(Element &e, Element &e2, Vector &f)
{
  static Vector eforce(3), cforce(3), sforce(3);
  static Vector tmpf(3);

  for(list<Connection *>::iterator it = e.connections.begin();
      it != e.connections.end(); it++)
  {
    Connection &c = **it;
    
    if(&e2 == c.element1 || &e2 == c.element2)
    {
      force(c, e, eforce);
    }
  }	  
  
  f = eforce;
}

void MechanicsModel::force(Connection &c, Element &e, Vector &f)
{
  f = 0;

  Element &e1 = *(c.element1);
  Element &e2 = *(c.element2);

  static Vector r(3), vrel(3), vrelNormal(3), vrelParallel(3),
    frictionForce(3), f0Normal(3), tmp(3);

  double rnorm, vrelnorm;
  double k, b, friction;

  //double cellSize = 2 * min(e1.radius, e2.radius);
  //k = (e1.material->young + e2.material->young) / 2 * cellSize;
  k = hooke(c);

  //std::cerr << "k: " << k << endl;

  b = damp(c);

  //std::cerr << "b: " << b << endl;

  friction = (e1.material->friction +
              e2.material->friction) / 2;
  

//   cout << "e1.position: " << endl;
//   e1.position.disp();

//   cout << "e2.position: " << endl;
//   e2.position.disp();

  r = e1.position;
  r.axpy(-1, e2.position);

//   cout << "r: " << endl;
//   r.disp();
  
  vrel = e1.velocity;
  vrel.axpy(-1, e2.velocity);
  
  rnorm = r.norm();
  vrelnorm = vrel.norm();
  
//   cout << "rnorm: " << rnorm << endl;

  // Hooke
  
  f = r;
  f *= -k * (rnorm - c.length) / rnorm;
  
  // Damping
  
  vrelParallel = r;
  vrelParallel *= (vrel * r) / (rnorm * rnorm);

  if(b > 0.0)
  {
    
    f.axpy(-b, vrelParallel);
  }

//   cout << "f: " << endl;
//   f.disp();

//   c.tension = f.norm();


  // Friction
  
  vrelNormal = vrel;
  vrelNormal.axpy(-1, vrelParallel);

  //if(friction > 0 && c.transient && rnorm < c.length &&
  // vrelNormal.norm() > 0)
  if(friction > 0 && c.transient && c.tension < 0)
  {
    frictionForce = vrelNormal;
    //    frictionForce *= 1.0 / vrelNormal.norm();
//     frictionForce *= -friction * f0Normal.norm();
    frictionForce *= -friction * f.norm();
//     frictionForce *= -friction * c.tension0;
    //frictionForce *= -friction;
    
    f.axpy(1, frictionForce);

  }

//   if((r * f) > 0)
//     c.tension = -2 * f.norm() / c.length;
//   else
//     c.tension = 2 * f.norm() / c.length;



  if(&e != &e1)
    f *= -1;
  
}

void MechanicsModel::plasticity(Connection &c)
{
  /*
  Element &e1 = *(c.element1);
  Element &e2 = *(c.element2);

  Vector r(3);
  double rnorm;
  
  add(e1.position, scaled(e2.position, -1), r);
  
  rnorm = two_norm(r);

  double plasticity = (e1.material->plasticity + e2.material->plasticity) / 2;

  if(rnorm > (1 + plasticity) * c.length)
  {
  std::cerr << "plasticity: " << rnorm / ((1 + plasticity) * c.length) << endl;
  
  c.damage *= rnorm / ((1 + plasticity) * c.length);
  c.length *= rnorm / ((1 + plasticity) * c.length);

  std::cerr << "damage: " << c.damage << endl;
  }
  */
}


double MechanicsModel::hooke(Connection &c)
{
  Element &e1 = *(c.element1);
  Element &e2 = *(c.element2);

  double cellSize = 2 * min(e1.radius, e2.radius);
  //double cellSize = (e1.radius + e2.radius);

  return (e1.material->young + e2.material->young) / 2 * cellSize;
}

double MechanicsModel::damp(Connection &c)
{
  Element &e1 = *(c.element1);
  Element &e2 = *(c.element2);

  double cellSize = 2 * min(e1.radius, e2.radius);
  //double cellSize = (e1.radius + e2.radius);

  return (e1.material->damp + e2.material->damp) / 2 * cellSize;
}

double MechanicsModel::mass(Element &e)
{
  return e.material->density * pow(2 * e.radius, 3);
}

}

