#include <algorithm>

#include <strstream>
#include <iostream>
#include <iomanip>

#include "ko/Physical.hpp"

#include "ko/PhysicalWatcher.hpp"
#include "ko/PhysicalManipulator.hpp"

using namespace std;

namespace ko
{

void Physical::addElement(Element &e)
{
  elements.push_back(&e);

  for(list<PhysicalWatcher *>::iterator it = watchers.begin();
      it != watchers.end(); it++)
  {
    PhysicalWatcher *w = *it;
    
    w->elementAdded(e);
  }
}

void Physical::removeElement(Element &e)
{
  // Need to implement some kind of dependency (connection) resolver

  list<Connection *> removals;

  for(list<Connection *>::iterator it = e.connections.begin();
      it != e.connections.end(); it++)
  {
    Connection &c = **it;
    
    removals.push_back(&c);
  }

  for(list<Connection *>::iterator it = removals.begin();
      it != removals.end(); it++)
  {
    Connection &c = **it;

    removeConnection(c);
  }

  elements.remove(&e);

  for(list<PhysicalWatcher *>::iterator it = watchers.begin();
      it != watchers.end(); it++)
  {
    PhysicalWatcher *w = *it;
    
    w->elementRemoved(e);
  }
  
  delete &e;
}

void Physical::addConnection(Connection &c)
{
  connections.push_back(&c);

  for(list<PhysicalWatcher *>::iterator it = watchers.begin();
      it != watchers.end(); it++)
  {
    PhysicalWatcher *w = *it;
    
    w->connectionAdded(c);
  }
}

void Physical::removeConnection(Connection &c)
{
  connections.remove(&c);

  for(list<PhysicalWatcher *>::iterator it = watchers.begin();
      it != watchers.end(); it++)
  {
    PhysicalWatcher *w = *it;
    
    w->connectionRemoved(c);
  }
  
  c.disconnect();
  delete &c;
}

void Physical::manipulate()
{
  for(list<PhysicalManipulator *>::iterator it = manipulators.begin();
      it != manipulators.end(); it++)
  {
    PhysicalManipulator *w = *it;
    
    w->manipulate(*this);
  }
}


void Physical::merge(Physical &p)
{
  for(list<Element *>::iterator it = p.elements.begin();
      it != p.elements.end(); it++)
  {
    Element &e = **it;
    
    addElement(e);
  }

  for(list<Connection *>::iterator it = p.connections.begin();
      it != p.connections.end(); it++)
  {
    Connection &c = **it;
    
    addConnection(c);
  }

  for(list<Material *>::iterator it = p.materials.begin();
      it != p.materials.end(); it++)
  {
    Material &m = **it;
    
    // O(n^2), expect materials to be few

    bool found = false;

    for(list<Material *>::iterator it2 = materials.begin();
	it2 != materials.end(); it2++)
    {
      Material &m2 = **it2;
      
      if(m.name == m2.name)
	found = true;
    }

    if(!found)
    {
      materials.push_back(&m);
    }
  }
}


int Physical::dof()
{
  int n = 0;

  for(list<Element *>::iterator it = elements.begin();
      it != elements.end(); it++)
  {
    Element &e = **it;
      
    if(!e.artificial)
      n++;
  }

  return n * 6;
}



void print(Physical &p)
{
  cout << "# Elements: " << p.elements.size() << endl;
  for(list<Element *>::iterator i = p.elements.begin();
      i != p.elements.end(); i++)
    {
      Element &e = **i;

      print(e);
    }

  cout << "# Connections: " << p.connections.size() << endl;
  for(list<Connection *>::iterator i = p.connections.begin();
      i != p.connections.end(); i++)
    {
      Connection &c = **i;

      print(c);
    }
}

void generateNames(Physical &p)
{
  int counter = 0;

  for(list<Element *>::iterator i = p.elements.begin();
      i != p.elements.end(); i++)
    {
      Element &e = **i;

      if(e.name == "")
	{
	  ostrstream data;

	  data << "element " << counter << ends;

	  e.name = data.str();
	  ++counter;

	}
    }
}



}

