#include <iostream>

#include "ko/SimpleVector.hpp"

using namespace std;

namespace ko
{

  /// Empty vector
  SimpleVector::SimpleVector() : n(3)
  {
    // Do nothing

    for(int i = 0; i < 3; i++)
    {
      values[i] = 0;
    }
  }
  
  /// Create vector of given size
  SimpleVector::SimpleVector(uint size) : n(3)
  {
    if(n != 3)
    {
      cerr << "SimpleVector: only vectors of size 3 are supported" << endl;
      dolfin::dolfin_segfault();
    }

    for(int i = 0; i < 3; i++)
    {
      values[i] = 0;
    }
  }

  /// Copy constructor
  SimpleVector::SimpleVector(const SimpleVector& x)
  {
    for(int i = 0; i < 3; i++)
    {
      values[i] = x.values[i];
    }
    n = x.n;
  }
  
  /// Destructor
  SimpleVector::~SimpleVector()
  {
    // Do nothing
  }

  dolfin::real SimpleVector::size() const
  {
    return n;
  }

  /// Assignment of vector
  const SimpleVector& SimpleVector::operator= (const SimpleVector& x)
  {
    for(int i = 0; i < 3; i++)
    {
      values[i] = x.values[i];
    }
    return *this;
  }
  
  /// Addition (AXPY)
  void SimpleVector::SimpleVector::axpy(const dolfin::real a,
					const SimpleVector& x)
  {
    for(int i = 0; i < 3; i++)
    {
      values[i] += a * x.values[i];
    }
  }

  /// Element access operator
  dolfin::real& SimpleVector::operator() (uint i)
  {
    return values[i];
  }

  /// Assignment of all elements to a single scalar value
  const SimpleVector& SimpleVector::operator= (dolfin::real a)
  {
    for(int i = 0; i < 3; i++)
    {
      values[i] = a;
    }
    return *this;
  }
  
  /// Multiply vector with scalar
  const SimpleVector& SimpleVector::operator*= (dolfin::real a)
  {
    for(int i = 0; i < 3; i++)
    {
      values[i] *= a;
    }
    return *this;
  }
  
  /// Add vector x
  const SimpleVector& SimpleVector::operator+= (const SimpleVector& x)
  {
    axpy(1, x);
  }

  /// Scalar product
  dolfin::real SimpleVector::operator*(const SimpleVector& x) const
  {
    dolfin::real sum = 0;
    for(int i = 0; i < 3; i++)
    {
      sum += values[i] * x.values[i];
    }
    return sum;
  }

  /// Compute norm of vector
  dolfin::real SimpleVector::norm(NormType type) const
  {
    dolfin::real result = 0;
    if(type == l2)
    {
      result = sqrt(operator*(*this));
    }
    else if(type == linf)
    {
      result = values[1];
      
      for(int i = 0; i < 3; i++)
      {
	result = max(result, values[i]);
      }
    }
    else if(type == l1)
    {
      cerr << "SimpleVector: unsupported norm type l1" << endl;
      dolfin::dolfin_segfault();
    }
    else
    {
      cerr << "SimpleVector: unsupported norm type" << endl;
      dolfin::dolfin_segfault();
    }
    return result;
  }
  
  /// Display vector
  void SimpleVector::disp() const
  {
    cerr << "SimpleVector: disp():" << endl;
    for(int i = 0; i < 3; i++)
    {
      cerr << values[i] << " ";
    }
    cerr << endl;
  }
}
