#if !defined(SIMPLEVECTOR_HPP)
#define SIMPLEVECTOR_HPP

#include <dolfin.h>

//using namespace std;
namespace ko
{

class SimpleVector
{
public:
  /// Empty vector
  SimpleVector();
  
  /// Create vector of given size
  SimpleVector(uint size);
  
  /// Copy constructor
  SimpleVector(const SimpleVector& x);
  
  /// Destructor
  ~SimpleVector ();
  
  /// Assignment of vector
  const SimpleVector& operator= (const SimpleVector& x);
  
  // Size
  dolfin::real size() const;
  
  /// Addition (AXPY)
  void axpy(const dolfin::real a, const SimpleVector& x);
  
  /// Element access operator
  dolfin::real& operator() (uint i);
  
  /// Assignment of all elements to a single scalar value
  const SimpleVector& operator= (dolfin::real a);
  
  /// Multiply vector with scalar
  const SimpleVector& operator*= (dolfin::real a);
  
  /// Add vector x
  const SimpleVector& operator+= (const SimpleVector& x);
  
  /// Scalar product
  dolfin::real operator*(const SimpleVector& x) const;
  
  /// Compute norm of vector
  enum NormType { l1, l2, linf };
  dolfin::real norm(NormType type = l2) const;
  
  /// Display vector
  void disp() const;
  
  
protected:
  dolfin::real values[3];
  uint n;
};

}

#endif // SIMPLEVECTOR_HPP
