#if !defined(BASICEULERINTEGRATOR_HPP)
#define BASICEULERINTEGRATOR_HPP

#include <list>
#include <map>
#include <set>
#include "ko/LinAlg.hpp"
#include "ko/Element.hpp"
#include "ko/Connection.hpp"
#include "ko/Physical.hpp"

#include "ko/NumericalIntegrator.hpp"
#include "ko/MechanicsModel.hpp"

namespace ko
{

class BasicEulerIntegrator : public NumericalIntegrator
{
public:

  enum integration_method {FORWARD_EULER=13, BACKWARD_EULER};

  enum integration_order {DG0 = 27, CG1};

  std::list<Element *> dynamicElements;
  int n;


  double eulerTolerance;
  int eulerMaxIters;

  double newtonTolerance;
  int newtonMaxIters;

  double linearTolerance;
  int linearMaxIters;

  bool demandConvergence;

  integration_method method;
  integration_order order;

  LargeVector cG1u10;

  BasicEulerIntegrator(Physical &physical);
  bool integrate(double t0, double &t1);
  void residualFunction(LargeVector &residual, double stepsize);
  void residualFunction(LargeVector &residual, double stepsize, Element *e);
  void residualFunctiondG0(LargeVector &residual, double stepsize);
  void residualFunctioncG1(LargeVector &residual, double stepsize);
  void residualFunctiondG0(LargeVector &residual, double stepsize, Element *e);
  void residualFunctioncG1(LargeVector &residual, double stepsize, Element *e);

  bool newton(double stepsize);
  bool fixedPoint(double stepsize);

  void elementDependents(Element &e, std::set<Element *> &dependents);
  void jacobian(std::map<Element *, int> lookup, Matrix &J, double stepsize);
};

}

#endif // BASICEULERINTEGRATOR_HPP
