#if !defined(MULTIADAPTIVEINTEGRATOR_HPP)
#define MULTIADAPTIVEINTEGRATOR_HPP


#include <dolfin.h>

#include <list>
#include <map>
//#include <hash_map>
#include <set>

#include "ko/LinAlg.hpp"
#include "ko/Element.hpp"
#include "ko/Connection.hpp"
#include "ko/Physical.hpp"

#include "ko/MechanicsModel.hpp"
#include "ko/NumericalIntegrator.hpp"

namespace ko
{

  class MultiAdaptiveIntegrator : public NumericalIntegrator,
				  public dolfin::ODE
  {
  public:
    
    /// Constructor
    MultiAdaptiveIntegrator(Physical &physical, double duration);

    /// Integrate over given time interval
    bool integrate(double t0, double &t1);
    void interpolatePhysical(double t);
    
    /// Interface for multi-adaptive solver
    double u0(unsigned int i);
    double timestep(unsigned int i);
    double f(const dolfin::Vector& Ui, dolfin::real t, unsigned int i);

  private:
    
    void updateLookups();
    void toArray(dolfin::Vector &Ui);
    void fromArray(const dolfin::Vector &Ui);
    void fromArray(const dolfin::Vector &Ui, Element &e);  
    void computeSparsity(dolfin::Matrix &A);
    void evaluate(dolfin::real t, dolfin::Vector &U, dolfin::Vector &result);

    unsigned int n;

    dolfin::Vector U0, U1, k;
    dolfin::Function U;
    dolfin::TimeStepper *timestepper;
    dolfin::Matrix S;
    
    //std::map<int, Element *> elementIndexLookup;
    std::vector<Element *> elementIndexLookup;
    std::map<Element *, int> indexElementLookup;
    std::list<Element *> dynamicElements;

  };
  
}

#endif // NUMERICALINTEGRATOR_HPP
