#if !defined(SIMULATOR_HPP)
#define SIMULATOR_HPP

#include <list>
#include <map>
#include "ko/LinAlg.hpp"
#include "ko/Element.hpp"
#include "ko/Connection.hpp"
#include "ko/Physical.hpp"
#include "ko/MechanicsModel.hpp"
#include "ko/IntersectionDetector.hpp"
#include "ko/PhysicalWatcher.hpp"

#include "ko/NumericalIntegrator.hpp"
#include "ko/BasicEulerIntegrator.hpp"
#include "ko/MultiAdaptiveIntegrator.hpp"

namespace ko
{

struct Simulator
{
public:
  // internal

  //enum integration_method {FORWARD_EULER=13, BACKWARD_EULER};

  NumericalIntegrator *integrator;
  BasicEulerIntegrator euler;
  MultiAdaptiveIntegrator multiAdaptive;

  //MechanicsModel mmodel;
  IntersectionDetector idetector;

  double stepsize;
  double time;
  double duration;

  bool doCollisions;

  /*
  double eulerTolerance;

  double newtonTolerance;
  int newtonMaxIters;

  double linearTolerance;
  int linearMaxIters;

  bool demandConvergence;
  */

  //integration_method method;

  Physical &physical;

  std::list<Connection *> newConnections;

  Simulator(Physical &physical, double duration);
  void step();
  /*
  bool euler();
  void backwardFunction(Element &e1, Element &e2, Vector &residual);

  void jacobian(std::list<Element *> &aelements, map<Element *, int> lookup,
		Matrix &J);

  void jacobianAnalytical(std::list<Element *> &aelements, map<Element *, int> lookup, Matrix &J);

  void explicitPosition(Element &e);

  void explicitVelocity(Element &e);
  bool implicitVelocityAll();
  */
  bool collisions();
  void fractures();
  void plasticity();

  void insertPhysical(Physical &p);

  bool isCollision(Element &e1, Element &e2);

};

}

#endif // PHYSICAL_HPP

