#include <iostream>
#include <strstream>

#include "ko/GeometryFiller.hpp"
#include "ko/IntersectionDetector.hpp"

using namespace std;

namespace ko
{



GeometryFiller::GeometryFiller()
{
}

void GeometryFiller::init(GtsSurface *surface)
{
  bbTree = gts_bb_tree_surface(surface);

  probePoint = gts_point_new(gts_point_class(), 0, 0, 0);
}

void GeometryFiller::fill(double density, bool isCenter)
{
  //double radius = 1 / density / 2 * 1.2;

  double cellSize = 1 / density;

  double radius = cellSize / 2;

  cerr << "cellSize: " << cellSize << endl;

  GtsBBox *largestBox = (GtsBBox *)bbTree->data;

  Vector min(3), max(3), center(3), span(3);

  min(0) = largestBox->x1;
  min(1) = largestBox->y1;
  min(2) = largestBox->z1;

  max(0) = largestBox->x2;
  max(1) = largestBox->y2;
  max(2) = largestBox->z2;

  Vector perturbation(3);

  perturbation(0) = 1e-07;
  perturbation(1) = 1e-07;
  perturbation(2) = 1e-07;

  max.axpy(1, perturbation);
  min.axpy(-1, perturbation);

  center = min;
  center.axpy(1, max);
  center *= 0.5;

  span = max;
  span.axpy(-1, min);


  // some perturbations to make it easier for the geometric predicates

  //mtl::scale(center, 1.01);
  //mtl::scale(span, 0.99);

  int xcount = (int)floor(span(0) * density);
  int ycount = (int)floor(span(1) * density);
  int zcount = (int)floor(span(2) * density);

  //int xcount = (int)rint(span(0) * density);
  //int ycount = (int)rint(span(1) * density);
  //int zcount = (int)rint(span(2) * density);

  Material *material = new Material;

  material->name = "default";
  material->strength = 1.0;

  physical.materials.push_back(material);

  int counter = 0;

  for(int i = 0; i < xcount; i++)
  {
    for(int j = 0; j < ycount; j++)
    {
      for(int k = 0; k < zcount; k++)
      {
	Vector p(3);

	if(isCenter)
	{
	  p(0) = center(0) - span(0) / 2 + cellSize * i;
	  p(1) = center(1) - span(1) / 2 + cellSize * j;
	  p(2) = center(2) - span(2) / 2 + cellSize * k;
	}
	else
	{
	  p(0) = min(0) + radius + cellSize * i;
	  p(1) = min(1) + radius + cellSize * j;
	  p(2) = min(2) + radius + cellSize * k;
	}

	///*
	// We want to make sure we're filling up all the way to the boundary.
	// We sample at half the cell size, and if any of the samples are
	// positive we fill the cell.

	double offset = cellSize / 4;
	
	int refineCounter = 0;
	
	for(int i = -1; i < 2; i += 2)
	{
	  for(int j = -1; j < 2; j += 2)
	  {
	    for(int k = -1; k < 2; k += 2)
	    {
	      Vector p2(3);
	      
	      p2 = p;
	      
	      p2(0) += i * offset;
	      p2(1) += j * offset;
	      p2(2) += k * offset;
	      
	      if(inside(p2))
	      {
		refineCounter++;
	      }
	    }
	  }
	}
	
	cerr << refineCounter << endl;

	if(refineCounter > 0)
	{
	  Element &e = *(new Element);

	  cerr << "Created element: " <<
	    p(0) << " " <<
	    p(1) << " " <<
	    p(2) <<
	    endl;


	  e.position = p;

	  e.radius = radius;

	  e.material = material;

	  physical.addElement(e);

	  ostrstream name;

	  name << counter << ends;

	  e.name = name.str();

	  counter++;

	}
	//*/

	/*

        if(inside(p))
        {
          Element &e = *(new Element);

	  mtl::copy(p, e.position);

          e.radius = radius;

          e.material = material;

          physical.addElement(e);

          ostrstream name;

          name << counter << ends;

          e.name = name.str();

          counter++;
	}

	*/

      }
    }
  }

  // Weld

  weld(physical, sqrt(3.0) * cellSize);

}

bool GeometryFiller::inside(Vector &p)
{
  gts_point_set(probePoint, p(0), p(1), p(2));

  /*
  GSList *boxlist = 0;

  boxlist = gts_bb_tree_stabbed(bbTree, probePoint);

  for(GSList *iter = boxlist; iter != 0; g_slist_next(iter))
  {
    GtsBBox *box = (GtsBBox *)iter->data;
    GtsObject *obj = (GtsObject *)box->bounded;

    if(gts_object_is_from_class(obj, gts_triangle_class()))
    {
      GtsTriangle *triangle = (GtsTriangle *)obj;

      

    }
  }
  */

  return gts_point_is_inside_surface(probePoint, bbTree, false);
}

void GeometryFiller::weld(Physical &physical, double distance)
{
  for(list<Element *>::iterator it = physical.elements.begin();
      it != physical.elements.end(); it++)
  {
    Element &e = **it;

    e.radius /= 10000;
  }

  IntersectionDetector idetector;

  idetector.init(physical.elements);

  Element probeElement;

  probeElement.radius = distance;

  for(list<Element *>::iterator it = physical.elements.begin();
      it != physical.elements.end(); it++)
  {
    Element &e = **it;

    probeElement.position = e.position;

    list<Element *> intersections;

    idetector.intersectionList(probeElement, intersections);

    for(list<Element *>::iterator it2 = intersections.begin();
        it2 != intersections.end(); it2++)
    {
      Element &e2 = **it2;

      if(&e != &e2 && !e.isConnected(e2))
      {
        
        Connection &c = *(new Connection());
        
        c.connect(e, e2);
        
        Vector r(3);
        
	r = e2.position;
	r.axpy(-1, e.position);
        
        double rnorm = r.norm();

        c.length = rnorm;
        
        physical.addConnection(c);
      }
    }
  }  

  for(list<Element *>::iterator it = physical.elements.begin();
      it != physical.elements.end(); it++)
  {
    Element &e = **it;

    e.radius *= 10000;
  }

}



}
