#include <Inventor/SoPath.h>
#include <Inventor/nodes/SoNode.h>
#include <Inventor/nodes/SoShape.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/actions/SoSubAction.h>

#include "ko/InventorGTSTranslator.hpp"

using namespace std;


namespace ko
{

InventorGTSTranslator::InventorGTSTranslator()
{
  SoDB::init();
  curboxes = 0;
}

SoCallbackAction::Response InventorGTSTranslator::preCallback(void *data, SoCallbackAction *a, const SoNode *node)
{
  InventorGTSTranslator *ia = (InventorGTSTranslator *)data;
  ia->pre(data, a, node);
}

SoCallbackAction::Response InventorGTSTranslator::pre(void *data, SoCallbackAction *action, const SoNode *node)
{
  curpath = action->getCurPath()->copy();
  paths.insert(curpath);
}

SoCallbackAction::Response InventorGTSTranslator::postCallback(void *data, SoCallbackAction *a, const SoNode *node)
{
  InventorGTSTranslator *ia = (InventorGTSTranslator *)data;
  ia->post(data, a, node);
}

SoCallbackAction::Response InventorGTSTranslator::post(void *data, SoCallbackAction *, const SoNode *node)
{
}

void InventorGTSTranslator::processTriangleCallback(void *data, SoCallbackAction *a, const SoPrimitiveVertex *v1, const SoPrimitiveVertex *v2, const SoPrimitiveVertex *v3)
{
  InventorGTSTranslator *ia = (InventorGTSTranslator *)data;
  ia->processTriangle(data, a, v1, v2, v3);
}

void InventorGTSTranslator::processTriangle(void *data, SoCallbackAction *action, const SoPrimitiveVertex *v1, const SoPrimitiveVertex *v2, const SoPrimitiveVertex *v3)
{
  SbVec3f p1 = v1->getPoint();
  SbVec3f p2 = v2->getPoint();
  SbVec3f p3 = v3->getPoint();

  // convert from object space to world space
  const SbMatrix & mm = action->getModelMatrix();
  mm.multVecMatrix(p1, p1);
  mm.multVecMatrix(p2, p2);
  mm.multVecMatrix(p3, p3);

  GtsVertex *tv1, *tv2, *tv3;
  GtsEdge *te1_2, *te2_3, *te3_1;
  GtsTriangle *curtri;

  tv1 = gts_vertex_new(gts_vertex_class(), p1[0], p1[1], p1[2]);
  tv2 = gts_vertex_new(gts_vertex_class(), p2[0], p2[1], p2[2]);
  tv3 = gts_vertex_new(gts_vertex_class(), p3[0], p3[1], p3[2]);

  te1_2 = gts_edge_new(gts_edge_class(), tv1, tv2);
  te2_3 = gts_edge_new(gts_edge_class(), tv2, tv3);
  te3_1 = gts_edge_new(gts_edge_class(), tv3, tv1);
  
  curtri = gts_triangle_new(gts_triangle_class(), te1_2, te2_3, te3_1);

  triangles.push_back(curtri);
}

void InventorGTSTranslator::translate(SoPath *path, GtsSurface **s)
{
  SoCallbackAction myAction;

  myAction.addPreCallback(SoShape::getClassTypeId(), 
			       preCallback, this);
  myAction.addPostCallback(SoShape::getClassTypeId(), 
			       postCallback, this);
  myAction.addTriangleCallback(SoShape::getClassTypeId(), 
			       processTriangleCallback, this);

  myAction.apply(path);

  generateSurface();

  *s = surface;

}

void InventorGTSTranslator::generateSurface()
{
  surface = gts_surface_new (gts_surface_class(),
			     gts_face_class(),
			     gts_edge_class(),
			     gts_vertex_class());

  for(list<GtsTriangle *>::iterator it = triangles.begin();
      it != triangles.end(); it++)
  {
    GtsTriangle *triangle = *it;

    GtsVertex *v1, *v2, *v3;
    GtsEdge *e1, *e2, *e3;

    gts_triangle_vertices_edges(triangle, NULL, &v1, &v2, &v3, &e1, &e2, &e3);

    gts_surface_add_face(surface, gts_face_new(gts_face_class(), e1, e2, e3));
  }  
}

void InventorGTSTranslator::write(FILE *fp, GtsSurface *s)
{
  gts_surface_write(s, fp);
}

}
