#if !defined(INVENTORGTSTRANSLATOR_HPP)
#define INVENTORGTSTRANSLATOR_HPP

#include <map>
#include <set>
#include <list>
#include <string>

#include <Inventor/SoPath.h>
#include <Inventor/nodes/SoNode.h>
#include <Inventor/actions/SoSubAction.h>
#include <Inventor/actions/SoCallbackAction.h>

#include <gts.h>

namespace ko
{

class InventorGTSTranslator
{
public:
  InventorGTSTranslator();


  static SoCallbackAction::Response preCallback(void *data, 
						  SoCallbackAction *,
						  const SoNode *node);
  SoCallbackAction::Response pre(void *data, 
				 SoCallbackAction *,
				 const SoNode *node);
  static SoCallbackAction::Response postCallback(void *data, 
						 SoCallbackAction *,
						 const SoNode *node);
  SoCallbackAction::Response post(void *data, 
				  SoCallbackAction *,
				  const SoNode *node);
  static void processTriangleCallback(void *, SoCallbackAction *,
				      const SoPrimitiveVertex *v1,
				      const SoPrimitiveVertex *v2,
				      const SoPrimitiveVertex *v3);
  void processTriangle(void *, SoCallbackAction *,
		       const SoPrimitiveVertex *v1,
		       const SoPrimitiveVertex *v2,
		       const SoPrimitiveVertex *v3);
  static void processLineSegmentCallback(void *, SoCallbackAction *,
					 const SoPrimitiveVertex *v1,
					 const SoPrimitiveVertex *v2);
  void processLineSegment(void *, SoCallbackAction *,
			  const SoPrimitiveVertex *v1,
			  const SoPrimitiveVertex *v2);

  void translate(SoPath *path, GtsSurface **s);

  void write(FILE *fp, GtsSurface *s);

protected:
  SoPath *curpath;  
  std::set<SoPath *> paths;

  GSList *curboxes;
  std::set<GNode *> pathtrees;

  std::list<GtsTriangle *> triangles;

  GtsSurface *surface;

  void generateSurface();
};

}

#endif
