#include <fstream>
#include <iostream>
#include <iomanip>

#include <dolfin.h>
//#include <dolfin/Mesh.h>

#include "ko/PhysicalDOMTranslator.hpp"
#include "ko/Physical.hpp"
#include "ko/Material.hpp"
#include "ko/Element.hpp"

using namespace ko;
using namespace std;

bool validIndex(int ix, int iy, int iz, int Nx, int Ny, int Nz)
{
  return ix != -1 && ix < Nx &&
    iy != -1 && iy < Ny &&
    iz != -1 && iz < Nz;
}

void createCell(dolfin::Mesh &m, dolfin::Node ****nodeArray,
		int n1x, int n1y, int n1z,
		int n2x, int n2y, int n2z,
		int n3x, int n3y, int n3z,
		int n4x, int n4y, int n4z,
		int Nx, int Ny, int Nz)
{
  if(validIndex(n1x, n1y, n1z, Nx, Ny, Nz) &&
     validIndex(n2x, n2y, n2z, Nx, Ny, Nz) &&
     validIndex(n3x, n3y, n3z, Nx, Ny, Nz) &&
     validIndex(n4x, n4y, n4z, Nx, Ny, Nz))
  {
    dolfin::Node &n1 = *(nodeArray[n1x][n1y][n1z]);
    dolfin::Node &n2 = *(nodeArray[n2x][n2y][n2z]);
    dolfin::Node &n3 = *(nodeArray[n3x][n3y][n3z]);
    dolfin::Node &n4 = *(nodeArray[n4x][n4y][n4z]);
	  
    dolfin::Cell &c = m.createCell(n1, n2, n3, n4);
  }
}

void generate(dolfin::Mesh &m, const int Nx, const int Ny, const int Nz)
{
  //const int Nx = 17, Ny = 5, Nz = 5;
  //const double hx = 1.0 / 99.0, hy = 1.0 / 99.0, hz = 1.0 / 99.0;
  const double hx = 0.15, hy = 0.15, hz = 0.15;


  //dolfin::Node *nodeArray[Nx][Ny][Nz];
  dolfin::Node**** nodeArray;

  nodeArray = new dolfin::Node***[Nx];

  for(int ix = 0; ix < Nx; ix++)
  {
    nodeArray[ix] = new dolfin::Node**[Ny];
    for(int iy = 0; iy < Ny; iy++)
    {
      nodeArray[ix][iy] = new dolfin::Node*[Nz];
    }
  }


  for(int ix = 0; ix < Nx; ix++)
  {
    for(int iy = 0; iy < Ny; iy++)
    {
      for(int iz = 0; iz < Nz; iz++)
      {
	nodeArray[ix][iy][iz] = 0;
      }
    }
  }
  
  for(int ix = 0; ix < Nx; ix++)
  {
    for(int iy = 0; iy < Ny; iy++)
    {
      for(int iz = 0; iz < Nz; iz++)
      {
	dolfin::Node &n = m.createNode(0, 0, 0);
	
	n.coord().x = hx * ix;
	n.coord().y = hy * iy;
	n.coord().z = hz * iz;
	
	nodeArray[ix][iy][iz] = &n;
      }
    }
  }
  
  
  for(int ix1 = 0; ix1 < Nx; ix1++)
  {
    for(int iy1 = 0; iy1 < Ny; iy1++)
    {
      for(int iz1 = 0; iz1 < Nz; iz1++)
      {
	int pos = (ix1 % 2) + (iy1 % 2) + (iz1 % 2);

	std::cerr <<
	  "pos: " << pos << " " <<
	  "ix1: " << ix1 << " " <<
	  "iy1: " << iy1 << " " <<
	  "iz1: " << iz1 <<
	  endl;

	if((pos % 2 == 0))
	{
	  // We generate a diamond shape of tetrahedrons around (ix1, iy1, iz1)

	  // Top of pyramid
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 - 1,
		     ix1 + 0, iy1 + 1, iz1 + 0,
		     Nx, Ny, Nz);
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 - 1,
		     ix1 - 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 1, iz1 + 0,
		     Nx, Ny, Nz);
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 - 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 + 1,
		     ix1 + 0, iy1 + 1, iz1 + 0,
		     Nx, Ny, Nz);
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 + 1,
		     ix1 + 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 1, iz1 + 0,
		     Nx, Ny, Nz);
	  
	  // Bottom of pyramid
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 - 1,
		     ix1 + 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 - 1, iz1 + 0,
		     Nx, Ny, Nz);
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 - 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 - 1,
		     ix1 + 0, iy1 - 1, iz1 + 0,
		     Nx, Ny, Nz);
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 + 1,
		     ix1 - 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 - 1, iz1 + 0,
		     Nx, Ny, Nz);
	  
	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 1, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 0, iz1 + 1,
		     ix1 + 0, iy1 - 1, iz1 + 0,
		     Nx, Ny, Nz);
	}

	if(pos == 3)
	{
	  ///*
	  // We fill in the holes between the diamonds around (ix1, iy1, iz1)

	  // Upper part (rotate ccw around y axis)

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 1, iy1 + 0, iz1 + 1,
		     ix1 + 1, iy1 + 1, iz1 + 0,
		     ix1 + 0, iy1 + 1, iz1 + 1,
		     Nx, Ny, Nz);

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 1, iy1 + 1, iz1 + 0,
		     ix1 + 1, iy1 + 0, iz1 - 1,
		     ix1 + 0, iy1 + 1, iz1 - 1,
		     Nx, Ny, Nz);

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 1, iz1 - 1,
		     ix1 - 1, iy1 + 0, iz1 - 1,
		     ix1 - 1, iy1 + 1, iz1 + 0,
		     Nx, Ny, Nz);

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 0, iy1 + 1, iz1 + 1,
		     ix1 - 1, iy1 + 1, iz1 + 0,
		     ix1 - 1, iy1 + 0, iz1 + 1,
		     Nx, Ny, Nz);

	  // Lower part (rotate ccw around y axis)

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 1, iy1 - 1, iz1 + 0,
		     ix1 + 1, iy1 + 0, iz1 + 1,
		     ix1 + 0, iy1 - 1, iz1 + 1,
		     Nx, Ny, Nz);

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 + 1, iy1 + 0, iz1 - 1,
		     ix1 + 1, iy1 - 1, iz1 + 0,
		     ix1 + 0, iy1 - 1, iz1 - 1,
		     Nx, Ny, Nz);

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 - 1, iy1 + 0, iz1 - 1,
		     ix1 + 0, iy1 - 1, iz1 - 1,
		     ix1 - 1, iy1 - 1, iz1 + 0,
		     Nx, Ny, Nz);

	  createCell(m, nodeArray,
		     ix1 + 0, iy1 + 0, iz1 + 0,
		     ix1 - 1, iy1 - 1, iz1 + 0,
		     ix1 + 0, iy1 - 1, iz1 + 1,
		     ix1 - 1, iy1 + 0, iz1 + 1,
		     Nx, Ny, Nz);
	  //*/
	}
      }
    }
  }
}

int main(int argC, char* argV[])
{
  dolfin::dolfin_output("plain text");

  int Nx = 9, Ny = 5, Nz = 5;

  dolfin::Mesh mesh;

  if(argC > 4)
  {
    Nx = atoi(argV[2]);
    Ny = atoi(argV[3]);
    Nz = atoi(argV[4]);
  }

  generate(mesh, Nx, Ny, Nz);
  
  if(argC > 1)
  {
    dolfin::File f(argV[1]);

//     ofstream os;
//     os.open(argV[1], ios::out);
    
    f << mesh;
    //os << mesh;
  }
  else
  {
    dolfin::File f("mesh.xml.gz");

    f << mesh;
  }
  
  return 0;
}

