#include <fstream>
#include <iostream>
#include <iomanip>
#include "ko/PhysicalDOMTranslator.hpp"
#include "ko/Physical.hpp"
#include "ko/Material.hpp"
#include "ko/Element.hpp"

using namespace ko;
using namespace std;

void generate(Physical &p)
{
  Material &m = *(new Material());

  m.density = 8E3;
  m.young = 7E9;
  m.damp = 1E2;
  m.strength = 1E8;

  m.name = "material";

  p.materials.push_back(&m);

  const int Nx = 3, Ny = 2, Nz = 2;
  //const double hx = 1.0 / 99.0, hy = 1.0 / 99.0, hz = 1.0 / 99.0;
  const double hx = 0.5, hy = 0.5, hz = 0.5;


  Element *elementArray[Nx][Ny][Nz];

  for(int ix = 0; ix < Nx; ix++)
    {
      for(int iy = 0; iy < Ny; iy++)
	{
	  for(int iz = 0; iz < Nz; iz++)
	    {
	      elementArray[ix][iy][iz] = 0;
	    }
	}
    }

  for(int ix = 0; ix < Nx; ix++)
    {
      for(int iy = 0; iy < Ny; iy++)
	{
	  for(int iz = 0; iz < Nz; iz++)
	    {
	      Element &e = *(new Element());

	      //e.radius = 1.0 / 99.0 / 2.0;
	      e.radius = 0.05;
	      //e.mass = 1.0;
	      //e.hooke = 1E5;
	      //e.damp = 1E2;
	      //e.strength = 0;
	      e.position(0) = hx * ix;//  + 0.7;
	      e.position(1) = hy * iy;//  + 0.5;
	      e.position(2) = hz * iz;

	      e.material = &m;

	      p.elements.push_back(&e);
	      elementArray[ix][iy][iz] = &e;
	    }
	}
    }


  for(int ix = 0; ix < Nx; ix++)
    {
      for(int iy = 0; iy < Ny; iy++)
	{
	  for(int iz = 0; iz < Nz; iz++)
	    {
	      if(elementArray[ix][iy][iz] != 0)
		{
		  Element &e1 = *(elementArray[ix][iy][iz]);

		  // we iterate through the neighborhood of e1

		  for(int iix = -1; iix < 2; iix++)
		    {
		      for(int iiy = -1; iiy < 2; iiy++)
			{
			  for(int iiz = -1; iiz < 2; iiz++)
			    {
			      // nested loops are fun

			      if(ix + iix != -1 && ix + iix < Nx &&
				 iy + iiy != -1 && iy + iiy < Ny &&
				 iz + iiz != -1 && iz + iiz < Nz &&
				 !(iix == 0 && iiy == 0 && iiz == 0)) 
				{
				  Element &e2 = *(elementArray[ix + iix][iy + iiy][iz + iiz]);
				  if(!e1.isConnected(e2))
				  {
				    Connection &c = *(new Connection());
				    
				    Vector r(3);
				    double rnorm;
				    
				    r = e1.position;
				    r.axpy(-1, e2.position);
				    
				    rnorm = r.norm();
				    
				    c.connect(e1, e2);
				    //c.hooke = 1E5;
				    //c.damp = 1E2;
				    //c.strength = 1E10;
				    c.length = rnorm;
				    
				    p.connections.push_back(&c);
				  }
				}
			    }
			}
		    }
		}
	    }
	}
    }
  

  list<Vector *> positions;

  Vector min(3), max(3), center(3);

  for(list<Element *>::iterator it = p.elements.begin();
      it != p.elements.end(); it++)
  {
    Element &e = **it;

    positions.push_back(&(e.position));
  }

  bbox(positions, min, max);

  center = 0;
  //center.add(0.5, min);
  //center.add(0.5, max);

  for(list<Element *>::iterator it = p.elements.begin();
      it != p.elements.end(); it++)
  {
    Element &e = **it;

    e.position.axpy(-1, center);
  }
}

int main(int argC, char* argV[])
{
  Physical p;
  PhysicalDOMTranslator t;
  
  generate(p);
  generateNames(p);

  if(argC > 1)
    {
      ofstream os;
      os.open(argV[1], ios::out);
      
      t.write(os, p);
    }
  else
    {
      t.write(cout, p);
    }

  return 0;
}
