#include <iostream>
#include <stdio.h>

#include "ko/PhysicalInventorInterface.hpp"

using namespace ko;
using namespace std;

PhysicalInventorInterface::PhysicalInventorInterface()
{
  SoDB::init();

  rootSeparator = new SoSeparator;

  rootSeparator->ref();

  elementsSeparator = new SoSeparator;
  connectionsSeparator = new SoSeparator;

  //elementMaterial = new SoMaterial;

  //elementMaterial->transparency.setValue(0.6);

  //elementsSeparator->addChild(elementMaterial);

  rootSeparator->addChild(elementsSeparator);
  rootSeparator->addChild(connectionsSeparator);
}

void PhysicalInventorInterface::addElement(Element &e)
{
  SoMaterial *elementMaterial = new SoMaterial();

  SoSeparator *elementSeparator = new SoSeparator();
  SoComplexity *elementComplexity = new SoComplexity();
  SoTransform *elementTransform = new SoTransform();
  SoSphere *elementSphere = new SoSphere();
  
  elementComplexity->value.setValue(0.2);

  elementSeparator->addChild(elementMaterial);
  elementSeparator->addChild(elementComplexity);
  elementSeparator->addChild(elementTransform);
  elementSeparator->addChild(elementSphere);
  elementsSeparator->addChild(elementSeparator);

  elementSeparatorMap[&e] = elementSeparator;
  elementTransformMap[&e] = elementTransform;
  elementSphereMap[&e] = elementSphere;
  elementMaterialMap[&e] = elementMaterial;

  sphereElementMap[elementSphere] = &e;
}

void PhysicalInventorInterface::removeElement(Element &e)
{
}

void PhysicalInventorInterface::addConnection(Connection &c)
{
  cerr << "PhysicalInventorInterface::addConnection" << endl;

  SoMaterial *connectionMaterial = new SoMaterial();
  
  SoSeparator *connectionSeparator = new SoSeparator();
  SoTransform *connectionTransform = new SoTransform();
  SoComplexity *connectionComplexity = new SoComplexity();
  SoCylinder *connectionCylinder = new SoCylinder();

  connectionComplexity->value.setValue(0.2);

  connectionSeparator->addChild(connectionMaterial);
  connectionSeparator->addChild(connectionComplexity);
  connectionSeparator->addChild(connectionTransform);
  connectionSeparator->addChild(connectionCylinder);
  
  connectionsSeparator->addChild(connectionSeparator);

  connectionSeparatorMap[&c] = connectionSeparator;
  connectionTransformMap[&c] = connectionTransform;
  connectionCylinderMap[&c] = connectionCylinder;
  connectionMaterialMap[&c] = connectionMaterial;
}

void PhysicalInventorInterface::removeConnection(Connection &c)
{
  connectionsSeparator->removeChild(connectionSeparatorMap[&c]);

  connectionSeparatorMap.erase(&c);
  connectionTransformMap.erase(&c);
  connectionCylinderMap.erase(&c);
  connectionMaterialMap.erase(&c);
}


void PhysicalInventorInterface::updateSceneGraph()
{
  for(list<Element *>::iterator it = addedElements.begin();
      it != addedElements.end(); it++)
  {
    Element &e = **it;

    addElement(e);
  }
  addedElements.clear();

  for(list<Connection *>::iterator it = addedConnections.begin();
      it != addedConnections.end(); it++)
  {
    Connection &c = **it;

    addConnection(c);
  }
  addedConnections.clear();

  for(list<Element *>::iterator it = removedElements.begin();
      it != removedElements.end(); it++)
  {
    Element &e = **it;

    removeElement(e);
  }
  removedElements.clear();

  for(list<Connection *>::iterator it = removedConnections.begin();
      it != removedConnections.end(); it++)
  {
    Connection &c = **it;

    removeConnection(c);
  }
  removedConnections.clear();


  for( map<Element *, SoTransform *>::iterator it =
	 elementTransformMap.begin(); it != elementTransformMap.end(); it++)
  {
    SoTransform *elementTransform = it->second;
    Element &e = *(it->first);

    elementTransform->translation.setValue(e.position(0),
					   e.position(1),
					   e.position(2));
  }

  for( map<Element *, SoSphere *>::iterator it =
	 elementSphereMap.begin(); it != elementSphereMap.end(); it++)
  {
    SoSphere *elementSphere = it->second;
    Element &e = *(it->first);

    elementSphere->radius.setValue(e.radius);
  }



  for( map<Element *, SoMaterial *>::iterator it =
	 elementMaterialMap.begin();
       it != elementMaterialMap.end(); it++)
  {
    SoMaterial *elementMaterial = it->second;
    Element &e = *(it->first);
    
    if(e.artificial)
    {
      elementMaterial->transparency.setValue(0.0);
      //elementMaterial->diffuseColor.setValue(1, 1, 1);
    }      
    else
    {
      elementMaterial->transparency.setValue(0.0);
    }
  }





  for( map<Connection *, SoTransform *>::iterator it =
	 connectionTransformMap.begin();
       it != connectionTransformMap.end(); it++)
  {
    SoTransform *connectionTransform = it->second;
    Connection &c = *(it->first);

    Element &e1 = *(c.element1);
    Element &e2 = *(c.element2);

    SbVec3f center((e1.position(0) + e2.position(0)) / 2,
		   (e1.position(1) + e2.position(1)) / 2,
		   (e1.position(2) + e2.position(2)) / 2);
    
    SbVec3f from(e1.position(0),
		 e1.position(1),
		 e1.position(2));
    
    SbVec3f to(e2.position(0),
	       e2.position(1),
	       e2.position(2));
    
    SbVec3f x(1, 0, 0);
    
    SbRotation xrot(x, M_PI / 2), temprot;

    connectionTransform->pointAt(from, to);
    
    temprot = connectionTransform->rotation.getValue();

    xrot *= temprot;

    connectionTransform->rotation.setValue(xrot);
    
    connectionTransform->translation.setValue(center);
  }


  for( map<Connection *, SoCylinder *>::iterator it =
	 connectionCylinderMap.begin();
       it != connectionCylinderMap.end(); it++)
  {
    SoCylinder *connectionCylinder = it->second;
    Connection &c = *(it->first);
    
    Element &e1 = *(c.element1);
    Element &e2 = *(c.element2);

    Vector r(3);
    double rnorm;
    
    r = e1.position;
    r.axpy(-1, e2.position);
    
    rnorm = r.norm();
    
    connectionCylinder->radius.setValue(min(e1.radius, e2.radius) / 5.0);
    connectionCylinder->height.setValue(rnorm);
  }  

  for( map<Connection *, SoMaterial *>::iterator it =
	 connectionMaterialMap.begin();
       it != connectionMaterialMap.end(); it++)
  {
    SoMaterial *connectionMaterial = it->second;
    Connection &c = *(it->first);
    
    Element &e1 = *(c.element1);
    Element &e2 = *(c.element2);


    double strength;
    
    if(c.transient)
    {
      cerr << "transient" << endl;
      strength = (c.element1->material->collisionStrength +
		  c.element2->material->collisionStrength) / 2 * c.length;
    }
    else
    {
      strength = (c.element1->material->strength +
		    c.element2->material->strength) / 2 * c.length;
    }
    
    if(!c.transient)
    {
      if(c.tension > 0)
      {
	min(3.0, 2.0);
	connectionMaterial->
	  diffuseColor.setValue(
				min(c.tension / strength, 1.0),
				0.0,
				0.5);
      }
      else
      {
	connectionMaterial->
	  diffuseColor.setValue(0.0,
				min(-c.tension / strength, 1.0),
				0.5);
      }
    }
    else
    {
      connectionMaterial->diffuseColor.setValue(0.0,
						0.0,
						0.5);
    }
    
  }
  
  //*/
}

