#if !defined(PHYSICALINVENTORINTERFACE_HPP)
#define PHYSICALINVENTORINTERFACE_HPP

#include <list>
#include <map>

#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoLabel.h>

#include "ko/LinAlg.hpp"
#include "ko/Element.hpp"
#include "ko/Connection.hpp"
#include "ko/Material.hpp"

using namespace ko;

class PhysicalInventorInterface
{
public:
  std::map<Element *, SoTransform *> elementTransformMap;
  std::map<Element *, SoSphere *> elementSphereMap;
  std::map<Element *, SoMaterial *> elementMaterialMap;
  std::map<Element *, SoSeparator *> elementSeparatorMap;

  std::map<SoSphere *, Element *> sphereElementMap;


  std::map<Connection *, SoTransform *> connectionTransformMap;
  std::map<Connection *, SoCylinder *> connectionCylinderMap;
  std::map<Connection *, SoMaterial *> connectionMaterialMap;
  std::map<Connection *, SoSeparator *> connectionSeparatorMap;

  SoSeparator *rootSeparator;
  SoSeparator *elementsSeparator;
  SoSeparator *connectionsSeparator;
  //SoMaterial *elementMaterial;

  std::list<Element *> addedElements, removedElements;
  std::list<Connection *> addedConnections, removedConnections;

  PhysicalInventorInterface();

  void addElement(Element &e);
  void removeElement(Element &e);

  void addConnection(Connection &c);
  void removeConnection(Connection &c);

  void updateSceneGraph();
};


#endif // PHYSICAL_HPP
