#include <iostream>

#include "ko/PhysicalDOMTranslator.hpp"
#include "ko/Physical.hpp"
#include "ko/Element.hpp"

using namespace ko;
using namespace std;
using namespace xercesc;

void writeElementPOV(Element &e)
{
  cout << "sphere{" << endl;
  cout << " <" << " " << 
    e.position(0) << ", " <<
    e.position(1) << ", " <<
    e.position(2) << ">," <<
    endl;
  cout << " " << e.radius << endl;
  cout << endl;
  cout << " texture{" << endl;
  if(e.artificial)
  {
    cout << "  pigment { color red 0.8 green 0.8 blue 0.8 filter 0.0}" << endl;
  }
  else
  {
    cout << "  pigment { color red 0.8 green 0.8 blue 0.8 filter 0.3}" << endl;
  }

  cout << "  finish { " <<
    "ambient " << 0.2 << " " <<
    "diffuse " << 0.6 << " " <<
    "specular " << 0.5 << " " <<
    "roughness " << 0.005 << " " <<
//     "ambient " << 0.1 << " " <<
//     "diffuse " << 0.1 << " " <<
//     "specular " << 0.1 << " " <<
//     "roughness " << 0.5 << " " <<
    "}" <<
    endl;

  //cout << "  sphereTexture" << endl;
  cout << " }" << endl;
  cout << "}" << endl;
  cout << endl;
}

void writeConnectionPOV(Connection &c)
{
  Element &e1 = *(c.element1);
  Element &e2 = *(c.element2);

  double strength;

  if(c.transient)
  {
//     cerr << "transient" << endl;
    strength = (c.element1->material->collisionStrength +
		c.element2->material->collisionStrength) / 2 * c.length;
  }
  else
  {
    strength = (c.element1->material->strength +
		c.element2->material->strength) / 2 * c.length;
  }
  

  cout << "cylinder{" << endl;
  cout << " <" << " " << 
    e1.position(0) << ", " <<
    e1.position(1) << ", " <<
    e1.position(2) << ">," <<
    endl;
  cout << " <" << " " <<
    e2.position(0) << ", " <<
    e2.position(1) << ", " <<
    e2.position(2) << ">," <<
    endl;
  cout << " " << min(e1.radius, e2.radius) / 5.0 << endl;
  cout << endl;
  cout << " texture{" << endl;
  //cout << "  sphereTexture" << endl;

  if(!c.transient)
  {
    if(c.tension > 0)
    {
      cout << "  pigment { color " <<
	//	"red " << c.tension / strength << " " <<
	"red " << 0.0 << " " <<
	"green " << 0.0 << " " <<
	"blue " << 0.5 << " " <<
	endl;
    }
    else
    {
      cout << "  pigment { color " <<
	"red " << 0.0 << " " <<
	//	"green " << -c.tension / strength << " " <<
	"green " << 0.0 << " " <<
	"blue " << 0.5 << " " <<
	endl;
    }
  }
  else
  {
      cout << "  pigment { color " <<
	"red " << 0.0 << " " <<
	"green " << 0.5 << " " <<
	"blue " << 0.5 << " " <<
	endl;
  }

  cout << "  }" << endl;


  cout << "  finish { " <<
    "ambient " << 0.2 << " " <<
    "diffuse " << 0.6 << " " <<
    "specular " << 0.5 << " " <<
    "roughness " << 0.005 << " " <<
    "}" <<
    endl;

  cout << " }" << endl;
  cout << "}" << endl;
  cout << endl;
}

void writePhysicalPOV(Physical &p)
{
  for(list<Element *>::iterator i = p.elements.begin();
      i != p.elements.end(); i++)
    {
      Element &e = **i;
      
      writeElementPOV(e);
    }

  double maxtension = 0;

  for(list<Connection *>::iterator i = p.connections.begin();
      i != p.connections.end(); i++)
    {
      Connection &c = **i;
      
      if(fabs(c.tension) > maxtension)
	maxtension = fabs(c.tension);
    }

  if(maxtension == 0)
    maxtension = 1;

  for(list<Connection *>::iterator i = p.connections.begin();
      i != p.connections.end(); i++)
    {
      Connection &c = **i;
      
      writeConnectionPOV(c);
    }

}

int main(int argC, char* argV[])
{
  dolfin::dolfin_output("silent");

  if(argC > 1)
    {
      Physical p;
      PhysicalDOMTranslator t;

      t.parse(argV[1], p);

      writePhysicalPOV(p);
    }

  return 0;
}

