#include <strstream>
#include <fstream>
#include <iomanip>



#include "ko/PhysicalDOMTranslator.hpp"
#include "ko/Simulator.hpp"
#include "ko/Physical.hpp"
#include "ko/Element.hpp"

#include "ko/PhysicalManipulator.hpp"

using namespace ko;
using namespace std;

ofstream gpos;

int main(int argC, char* argV[])
{
  std::string physicalfile, dolfinfile;

  dolfin::dolfin_output("plain text");

  dolfin::real T = 10.0;
  dolfin::real g = -9.81;
  std::string method = "nostab";

  bool output = true;

  if(argC == 1)
  {
    dolfin::dolfin_info("Usage: ko-simulator physical-file parameter-file T g");
    dolfin::dolfin_info("");
    dolfin::dolfin_info("physical-file - XML file with masses and springs");
    dolfin::dolfin_info("parameter-file - XML file with integration parameters");
    dolfin::dolfin_info("T - end time of simulation");
    dolfin::dolfin_info("g - gravitational acceleration");
    return 1;
  }


  if(argC > 1)
  {
    physicalfile = argV[1];
  }

  if(argC > 2)
  {
    dolfinfile = argV[2];

    dolfin::dolfin_load(dolfinfile.c_str());
    cerr << "foo" << endl;

    cerr << "tolerance: " << (double)dolfin::dolfin_get("tolerance") << endl;
    //cerr << "method: " << (char *)dolfin::dolfin_get("method") << endl;
  }

//   if(argC > 3)
//   {
//     std::string arg = argV[3];
//     if(arg == "stab")
//       method = arg;
//     if(arg == "newton")
//       method = arg;
//     if(arg == "nostab")
//       method = arg;

//   }

  cerr << "method: " << method << endl;

  if(argC > 3)
  {
    T = atof(argV[3]);

  }

  cerr << "T: " << T << endl;

  if(argC > 4)
  {
    g = atof(argV[4]);

  }

  cerr << "g: " << g << endl;

  Physical p;
  
  PhysicalDOMTranslator t;
  
  t.parse(physicalfile.c_str(), p);
  
  Simulator s(p, T);
  
  cerr << "foo" << endl;
  
  
  dolfin::real k = dolfin::dolfin_get("maximum time step");;

  dolfin::real ksmall = 1e-5;
  
  //s.integrator = &s.multiAdaptive;


//   s.doCollisions = false;
  s.doCollisions = true;
  
  dolfin::real dolfinTOL = dolfin::dolfin_get("tolerance");

  s.euler.demandConvergence = true;
  s.euler.eulerMaxIters = 100;
  //s.euler.eulerTolerance = 1E-6;
  s.euler.eulerTolerance = dolfinTOL * 0.01;

  s.stepsize = k;
  
  if(method == "stab")
  {
    s.integrator = &s.multiAdaptive;
  }
  if(method == "newton")
  {
    s.integrator = &s.euler;
    s.euler.method = BasicEulerIntegrator::BACKWARD_EULER;
  }
  if(method == "nostab")
  {
    s.integrator = &s.euler;
    s.euler.method = BasicEulerIntegrator::FORWARD_EULER;
  }

  std::string dolfinMethod = dolfin::dolfin_get("method");

  if(dolfinMethod == "cg")
  {
    s.euler.order = BasicEulerIntegrator::CG1;
  }
  else
  {
    s.euler.order = BasicEulerIntegrator::DG0;
  }
  //s.euler.order = BasicEulerIntegrator::DG0;

  //s.integrator->mmodel.g(0) = 1 * -9.81 / 1000.0;
  s.integrator->mmodel.g *= 0;
  
  s.integrator->mmodel.g(1) = g;
  //s.integrator->mmodel.g(1) = -1 * 9.81 / 1.0;
  
  
  
  int counter = 0;
  double lasttime = -1;
  
  while(s.time < s.duration)
  {
    ///*
    // 15 fps

    if(output)
    {
      while(s.time >= (counter * 0.03334))
      {
	s.integrator->interpolatePhysical(counter * 0.03334);
	
	ostrstream filename;
	
	filename.fill('0');
	filename.width(6);
	
	filename << counter << ".xml" << ends;
	
	char *fname = filename.str();
	
	cerr << "writing" << endl;
	cerr << fname << endl;
	
	ofstream os;
	os.open(fname, ios::out);
	
	t.write(os, s.physical);
	
	lasttime = s.time;
	
	counter++;
	os.close();
	
	delete fname;
      }
    }    
    
//     cerr << "connections: " << s.physical.connections.size() << endl;
    
    s.integrator->interpolatePhysical(s.time);
    //*/
    
    s.step();
    
    
    
  }
  
  return 0;
}
